% PrScript4SAXS.m (4 SAXS curve analysis of a 2 metal labeled compound)
% March, 2020, Daisuke Kohda (Kyushu University, Japan)
% credits to MATLAB Data Analysis Scripts Found at: doi:10.1371/journal.pone.0003229.s002 (2.42 MB ZIP)
% Supplementary Materials S2 of "A Molecular Ruler for Measuring Quantitative Distance Distributions"
% Rebecca S. Mathew-Fenn, Rhiju Das, Joshua A. Silverman, Peter A. Walker, Pehr A. B. Harbury
% PLos ONE 3: e3229 (2008)

% initial settings                <===================== min and max inputs
qvalmin=0.05; % min s = 0.008 A-1
qvalmax=0.4;  % max s = 0.064 A-1

% read SAXS data from Excel file
filename='I2-PEGn_SAXS_data.xlsx';
EDATA=xlsread(filename); % read in from the first sheet

% read Q and convert to s
svalmax=qvalmax/(2*pi);  
svalmin=qvalmin/(2*pi); 
sval=EDATA(:,1)./(2*pi);   

% read I and sigI
flag='PEG9';                  % <===================== select PEG3, PEG5, or PEG8
maintitle=sprintf('%s %4.3f<s<%4.3f %s %s', filename,svalmin,svalmax,flag,'Mar12_2020');

switch flag
    case 'PEG3' 
f=EDATA(:,2); 
sigI_f=EDATA(:,3);
u=EDATA(:,8);
sigI_u=EDATA(:,9);
u=u-f;

f=EDATA(:,2); 
sigI_f=EDATA(:,3);
a=EDATA(:,6);
sigI_a=EDATA(:,7);
a=a-f;

f=EDATA(:,2); 
sigI_f=EDATA(:,3);
b=EDATA(:,6);
sigI_b=EDATA(:,7);
b=b-f;

f=EDATA(:,2); 
sigI_f=EDATA(:,3);
ab=EDATA(:,4);
sigI_ab=EDATA(:,5);
ab=ab-f;

    case 'PEG5' 
f=EDATA(:,11); 
sigI_f=EDATA(:,12);
u=EDATA(:,17);
sigI_u=EDATA(:,18);
u=u-f;

f=EDATA(:,11); 
sigI_f=EDATA(:,12);
a=EDATA(:,15);
sigI_a=EDATA(:,16);
a=a-f;

f=EDATA(:,11); 
sigI_f=EDATA(:,12);
b=EDATA(:,15);
sigI_b=EDATA(:,16);
b=b-f;

f=EDATA(:,11); 
sigI_f=EDATA(:,12);
ab=EDATA(:,13);
sigI_ab=EDATA(:,14);
ab=ab-f;

    case 'PEG9' 
f=EDATA(:,20); 
sigI_f=EDATA(:,21);
u=EDATA(:,26);
sigI_u=EDATA(:,27);
u=u-f;

f=EDATA(:,20); 
sigI_f=EDATA(:,21);
a=EDATA(:,24);
sigI_a=EDATA(:,25);
a=a-f;

f=EDATA(:,20); 
sigI_f=EDATA(:,21);
b=EDATA(:,24);
sigI_b=EDATA(:,25);
b=b-f;

f=EDATA(:,20); 
sigI_f=EDATA(:,21);
ab=EDATA(:,22);
sigI_ab=EDATA(:,23);
ab=ab-f;

   otherwise
    disp('Unkown flag');
end


%noise estimation (simply add sigIs)
sigI_Idelta=sigI_ab+sigI_a+sigI_b+sigI_u;

% initialize figures
close all

% logical indexing
% idx=(sval<=svalmax);
idx=((sval>=svalmin) & (sval<=svalmax));
sval=sval(idx); f=f(idx); u=u(idx); a=a(idx);b=b(idx); ab=ab(idx);
sigI_Idelta=sigI_Idelta(idx);

% open figure
figure ('position',[100 400 1500 1200]);

% super tile
h=sgtitle(maintitle);
% disable printing underscore as subscript in figures
set(h, 'FontSize', 10,'Interpreter','none');

%plot SAXS curves
SAXSCurve=subplot(3,3,1);
hold on;
title('SAXS curves', 'FontSize',8);
xlabel('s (A-1)');
ylabel('Intensity');
Param={'u','a','b','ab','Idelta'};
plot(sval,u); plot(sval,a); plot(sval, b); plot(sval,ab);


% Idelta:=ab-(a+b)+u
opt1=1.0; opt2=1.0;
Idelta=ab-opt1.*(a+b)+opt2.*u;

% BC correction        <================ set offset manually if necessay

switch flag
    case 'PEG3' 
    bc=-0.41e-6;
    
    case 'PEG5' 
    bc=0.01e-6;

    case 'PEG9' 
    bc=-0.15e-6;
end

Idelta=Idelta+bc;

subplot(3,3,1);
plot(sval,Idelta);
legend(Param,'Location','Best');
hold off; 
% lsqnonneg
DBIN=1;
Dval=1:DBIN:200;
% element name        <======================== select element I or Tb
square_F=gen_square_Fiodo(sval);   % iodine
% square_F=gen_square_Ftb(sval);   % terbium
IqD=gen_IqDA(square_F,sval,Dval);

subplot(3,3,6);
plot(sval,IqD);
title('basis functions, IqD', 'FontSize',8);
xlabel('s (A-1)');
ylabel('Intensity (arbitrary units)');
% adjust TolX if lsqnonneg fails
% options=optimset('TolX',1e-4); % default
% options=optimset('Tolx',1e-3);
options=optimset('TolX',1e-2);
% options=optimset('TolX',1e-1);
PII=lsqnonneg(IqD,Idelta,options);

DvalPII=subplot(3,3,5);
plot(Dval,PII);
title('P(r) with lsqnonneg', 'FontSize', 8);
xlabel('distance (A)');
ylabel('fraction');

ExpSim=subplot(3,3,4);
hold on; plot(sval,Idelta); plot(sval,IqD*PII);
hold off;
title('SAXS curves, exp vs sim', 'FontSize',8);
xlabel('s (A-1)');
ylabel('Intensity');
Param={'experiment','simulation'};
legend(Param,'Location','Best');

% maximu entropy
%redefine Dval
DBIN=1;
Dval=1:DBIN:200;
% remake 
IqDfms=gen_IqDA(square_F,sval,Dval);

%noise level              
% Idelta_std=ones(length(sval),1)*1e-6;
Idelta_std=sigI_Idelta;

%generate ICF, multipler=3 for DNA
load mask.txt
ICF=gen_ICFA(Dval,mask,3);

% The basis function (IqD) must be scaled to be smaller
% than the magnitude of the ocillations in the intererence
% profile (= SAXS intensity curve)        
scaling=3e9;                   % <============== set scaling factor manually

% jacknife optimization or use a constant alpha
flag2='optimize';

switch flag2
    case 'no' 
%jacknife=repeat fms5 to estimate alpha_stop
 [fMAT, fvec, alphavec]=jacknifeA(sval,Idelta,Idelta_std,IqDfms./scaling,ICF);
 subplot(3,3,8);
 plot(Dval,fvec);
 title('P(r) with maximum entropy','FontSize',8);
 xlabel('distance (A)');
 ylabel('fraction');
 alpha_stop=mean(alphavec);
 
    otherwise              %  <============== set alpha manually if no optimization
        
        switch flag     
            case 'PEG3' 
            alpha_stop=6.2;
    
            case 'PEG5' 
            alpha_stop=2.8;

            case 'PEG9' 
            alpha_stop=12.5;
        end
 end
  
%fms5
 [hvec,fvec,sfvec,cscas,alpha]=fms5A(sval,Idelta,Idelta_std,IqDfms./scaling,ICF,alpha_stop,0.05,1,0)
 
% redraw figures
subplot(3,3,7);
hold off; hold on;
plot(sval,Idelta,'DisplayName','Idelta(exp)');
plot(sval,IqDfms./scaling*fvec, 'DisplayName','Identa(sim)');
plot(sval,sigI_Idelta,'DisplayName','+std');
plot(sval,-sigI_Idelta, 'DisplayName','-std');
hold off;
title('SAXS curves, exp vs sim with jacknife/fms5','FontSize',8);
xlabel('s (A-1)');
ylabel('Intensity');
%Param={'','','Idelta(exp)','Idelta(sim)','sigI-Idelta'};
%legend(Param,'Location','Best');
legend('show');

%write to an excel file
% headers={'S'; 'u'; 'a', 'ab', 'Idelta'; 'sigI_Idelta'; 'IqDfms'};
OUTPUT1=[sval u a ab Idelta sigI_Idelta IqDfms./scaling*fvec]; 
% writematrix(OUTPUT1, sprintf('%s_intens_%s',flag,filename)); 

subplot(3,3,8);
plot(Dval,fvec);
title('P(r) with maximum entropy with jacknife/fms5','FontSize',8);
xlabel('distance (A)');
ylabel('fraction');

%write to an excel file
% headers={'distance (A)'; 'P(r)'};
OUTPUT2=[Dval' fvec];
% writematrix(OUTPUT2, sprintf('%s_PairDist_%s',flag, filename)); 


subplot(3,3,9);
hold on
plot(Dval(1:length(Dval)/5),fvec(1:length(Dval)/5));
% errorbar(Dval(1:length(Dval)/5),fvec(1:length(Dval)/5),sfvec(1:length(Dval)/5));
% plot(Dval(1:length(Dval)/5),smmoth(fvec(1:length(Dval)/5),5));
title(['alpha =' num2str(alpha_stop) ',  BC =' num2str(bc) ],'FontSize',8);
xlabel('distance (A)');
ylabel('fraction');
hold off;











